@@ -1,31 +1,27 @@
-#############################################################################
-##
-## Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-## Contact: http://www.qt-project.org/legal
-##
-## This file is part of Qt Creator.
-##
-## Commercial License Usage
-## Licensees holding valid commercial Qt licenses may use this file in
-## accordance with the commercial license agreement provided with the
-## Software or, alternatively, in accordance with the terms contained in
-## a written agreement between you and Digia.  For licensing terms and
-## conditions see http://qt.digia.com/licensing.  For further information
-## use the contact form at http://qt.digia.com/contact-us.
-##
-## GNU Lesser General Public License Usage
-## Alternatively, this file may be used under the terms of the GNU Lesser
-## General Public License version 2.1 as published by the Free Software
-## Foundation and appearing in the file LICENSE.LGPL included in the
-## packaging of this file.  Please review the following information to
-## ensure the GNU Lesser General Public License version 2.1 requirements
-## will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-##
-## In addition, as a special exception, Digia gives you certain additional
-## rights.  These rights are described in the Digia Qt LGPL Exception
-## version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
-##
-#############################################################################
+############################################################################
+#
+# Copyright (C) 2016 The Qt Company Ltd.
+# Contact: https://www.qt.io/licensing/
+#
+# This file is part of Qt Creator.
+#
+# Commercial License Usage
+# Licensees holding valid commercial Qt licenses may use this file in
+# accordance with the commercial license agreement provided with the
+# Software or, alternatively, in accordance with the terms contained in
+# a written agreement between you and The Qt Company. For licensing terms
+# and conditions see https://www.qt.io/terms-conditions. For further
+# information use the contact form at https://www.qt.io/contact-us.
+#
+# GNU General Public License Usage
+# Alternatively, this file may be used under the terms of the GNU
+# General Public License version 3 as published by the Free Software
+# Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+# included in the packaging of this file. Please review the following
+# information to ensure the GNU General Public License requirements will
+# be met: https://www.gnu.org/licenses/gpl-3.0.html.
+#
+############################################################################
 
 import re
 
@@ -47,7 +43,7 @@ def modifyRunSettingsForHookInto(projectName, kitCount, port):
     result = __configureCustomExecutable__(projectName, port, mkspec, qtVersion)
     if result:
         ensureChecked(":RunSettingsEnvironmentDetails_Utils::DetailsButton")
-        envVarsTableView = waitForObject("{type='QTableView' visible='1' unnamed='1'}")
+        envVarsTableView = waitForObject("{type='QTreeView' visible='1' unnamed='1'}")
         model = envVarsTableView.model()
         changingVars = []
         for index in dumpIndices(model):
@@ -68,6 +64,11 @@ def modifyRunSettingsForHookInto(projectName, kitCount, port):
                     changingVars.append("SQUISH_LIBQTDIR=%s" % replacement)
                 else:
                     changingVars.append(varName)
+            elif varName == "DYLD_FRAMEWORK_PATH" and platform.system() == 'Darwin':
+                value = str(model.data(model.index(index.row, 1)).toString())
+                test.log("Adding %s to DYLD_FRAMEWORK_PATH" % qtLibPath)
+                replacement = ":".join(filter(len, [qtLibPath, value]))
+                changingVars.append("%s=%s" % (varName, replacement))
         batchEditRunEnvironment(kitCount, 0, changingVars, True)
     switchViewTo(ViewConstants.EDIT)
     return result
@@ -85,9 +86,9 @@ def batchEditRunEnvironment(kitCount, currentTarget, modifications, alreadyOnRun
     clickButton(waitForObject("{text='OK' type='QPushButton' unnamed='1' visible='1' "
                               "window=':Edit Environment_ProjectExplorer::EnvironmentItemsDialog'}"))
 
-def modifyRunSettingsForHookIntoQtQuickUI(kitCount, workingDir, projectName, port, quickVersion=1):
+def modifyRunSettingsForHookIntoQtQuickUI(kitCount, kit, workingDir, projectName, port, quickVersion="1.1"):
     switchViewTo(ViewConstants.PROJECTS)
-    switchToBuildOrRunSettingsFor(kitCount, 0, ProjectSettings.RUN, True)
+    switchToBuildOrRunSettingsFor(kitCount, kit, ProjectSettings.RUN, True)
 
     qtVersion, mkspec, qtLibPath, qmake = getQtInformationForQmlProject()
     if None in (qtVersion, mkspec, qtLibPath, qmake):
@@ -102,7 +103,7 @@ def modifyRunSettingsForHookIntoQtQuickUI(kitCount, workingDir, projectName, por
                      "Using fallback of pushing STOP inside Creator.")
         return None
     test.log("Using (QtVersion/mkspec) %s/%s with SquishPath %s" % (qtVersion, mkspec, squishPath))
-    if quickVersion == 1:
+    if quickVersion == "1.1":
         if platform.system() == "Darwin":
             executable = os.path.abspath(os.path.dirname(qmake) + "/QMLViewer.app")
         else:
@@ -178,10 +179,10 @@ def __getMkspecFromQMakeConf__(qmakeConf):
     return os.path.basename(mkspec)
 
 def __getMkspecFromQmake__(qmakeCall):
-    if getOutputFromCmdline("%s -query QT_VERSION" % qmakeCall).strip().startswith("5."):
-        return getOutputFromCmdline("%s -query QMAKE_XSPEC" % qmakeCall).strip()
+    if getOutputFromCmdline([qmakeCall, "-query", "QT_VERSION"]).strip().startswith("5."):
+        return getOutputFromCmdline([qmakeCall, "-query", "QMAKE_XSPEC"]).strip()
     else:
-        QmakeConfPath = getOutputFromCmdline("%s -query QMAKE_MKSPECS" % qmakeCall).strip()
+        QmakeConfPath = getOutputFromCmdline([qmakeCall, "-query", "QMAKE_MKSPECS"]).strip()
         for tmpPath in QmakeConfPath.split(os.pathsep):
             tmpPath = tmpPath + os.sep + "default" + os.sep +"qmake.conf"
             result = __getMkspecFromQMakeConf__(tmpPath)
@@ -223,17 +224,6 @@ def __configureCustomExecutable__(projectName, port, mkspec, qmakeVersion):
     __invokeAddCustomExecutable__(startAUT, args)
     return True
 
-# function that retrieves a specific child object by its class
-# this is sometimes the best way to avoid using waitForObject() on objects that
-# occur more than once - but could easily be found by using a compound object
-# (e.g. search for Utils::PathChooser instead of Utils::BaseValidatingLineEdit and get the child)
-def getChildByClass(parent, classToSearchFor, occurence=1):
-    children = [child for child in object.children(parent) if className(child) == classToSearchFor]
-    if len(children) < occurence:
-        return None
-    else:
-        return children[occurence - 1]
-
 # get the Squish path that is needed to successfully hook into the compiled app
 def getSquishPath(mkspec, qmakev):
     # assuming major and minor version will be enough
@@ -306,7 +296,7 @@ def deleteAppFromWinFW(workingDir, projectName, isReleaseBuild=True):
         test.warning("Could not delete %s as allowed program from win firewall" % (projectName))
 
 # helper that can modify the win firewall to allow a program to communicate through it or delete it
-# param addToFW defines whether to add (True) or delete (False) this programm to/from the firewall
+# param addToFW defines whether to add (True) or delete (False) this program to/from the firewall
 def __configureFW__(workingDir, projectName, isReleaseBuild, addToFW=True):
     if isReleaseBuild == None:
         if projectName[-4:] == ".exe":
@@ -326,7 +316,8 @@ def __configureFW__(workingDir, projectName, isReleaseBuild, addToFW=True):
     # Needs admin privileges on Windows 7
     # Using the deprecated "netsh firewall" because the newer
     # "netsh advfirewall" would need admin privileges on Windows Vista, too.
-    return subprocess.call('netsh firewall %s allowedprogram "%s.exe" %s %s' % (mode, path, projectName, enable))
+    return subprocess.call(["netsh", "firewall", mode, "allowedprogram",
+                            "%s.exe" % path, projectName, enable])
 
 # helper to check whether win firewall is running or not
 # this doesn't check for other firewalls!
@@ -336,18 +327,13 @@ def __isWinFirewallRunning__():
     if not platform.system() in ('Microsoft' 'Windows'):
         __isWinFirewallRunning__.fireWallState = False
         return False
-    result = getOutputFromCmdline("netsh firewall show state")
+    result = getOutputFromCmdline(["netsh", "firewall", "show", "state"])
     for line in result.splitlines():
         if "Operational mode" in line:
             __isWinFirewallRunning__.fireWallState = not "Disable" in line
             return __isWinFirewallRunning__.fireWallState
     return None
 
-def __fixQuotes__(string):
-    if platform.system() in ('Windows', 'Microsoft'):
-        string = '"' + string + '"'
-    return string
-
 # this function adds the given executable as an attachable AUT
 # Bad: executable/port could be empty strings - you should be aware of this
 def addExecutableAsAttachableAUT(executable, port, host=None):
@@ -358,8 +344,8 @@ def addExecutableAsAttachableAUT(executable, port, host=None):
     squishSrv = __getSquishServer__()
     if (squishSrv == None):
         return False
-    result = subprocess.call(__fixQuotes__('"%s" --config addAttachableAUT "%s" %s:%s')
-                             % (squishSrv, executable, host, port), shell=True)
+    result = subprocess.call([squishSrv, "--config", "addAttachableAUT",
+                              executable, "%s:%s" % (host, port)])
     if result == 0:
         test.passes("Added %s as attachable AUT" % executable)
     else:
@@ -376,8 +362,8 @@ def removeExecutableAsAttachableAUT(executable, port, host=None):
     squishSrv = __getSquishServer__()
     if (squishSrv == None):
         return False
-    result = subprocess.call(__fixQuotes__('"%s" --config removeAttachableAUT "%s" %s:%s')
-                             % (squishSrv, executable, host, port), shell=True)
+    result = subprocess.call([squishSrv, "--config", "removeAttachableAUT",
+                              executable, "%s:%s" % (host, port)])
     if result == 0:
         test.passes("Removed %s as attachable AUT" % executable)
     else:
@@ -403,14 +389,14 @@ def __invokeAddCustomExecutable__(exe, args, workingDir=None):
     addMenu = addButton.menu()
     activateItem(waitForObjectItem(addMenu, 'Custom Executable'))
     exePathChooser = waitForObject(":Executable:_Utils::PathChooser")
-    exeLineEd = getChildByClass(exePathChooser, "Utils::BaseValidatingLineEdit")
+    exeLineEd = getChildByClass(exePathChooser, "Utils::FancyLineEdit")
     argLineEd = waitForObject("{buddy={window=':Qt Creator_Core::Internal::MainWindow' "
                               "type='QLabel' text='Arguments:' visible='1'} type='QLineEdit' "
                               "unnamed='1' visible='1'}")
     wdPathChooser = waitForObject("{buddy={window=':Qt Creator_Core::Internal::MainWindow' "
                                   "text='Working directory:' type='QLabel'} "
                                   "type='Utils::PathChooser' unnamed='1' visible='1'}")
-    wdLineEd = getChildByClass(wdPathChooser, "Utils::BaseValidatingLineEdit")
+    wdLineEd = getChildByClass(wdPathChooser, "Utils::FancyLineEdit")
     replaceEditorContent(exeLineEd, exe)
     replaceEditorContent(argLineEd, args)
     if workingDir:
