@@ -1,41 +1,43 @@
-#############################################################################
-##
-## Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-## Contact: http://www.qt-project.org/legal
-##
-## This file is part of Qt Creator.
-##
-## Commercial License Usage
-## Licensees holding valid commercial Qt licenses may use this file in
-## accordance with the commercial license agreement provided with the
-## Software or, alternatively, in accordance with the terms contained in
-## a written agreement between you and Digia.  For licensing terms and
-## conditions see http://qt.digia.com/licensing.  For further information
-## use the contact form at http://qt.digia.com/contact-us.
-##
-## GNU Lesser General Public License Usage
-## Alternatively, this file may be used under the terms of the GNU Lesser
-## General Public License version 2.1 as published by the Free Software
-## Foundation and appearing in the file LICENSE.LGPL included in the
-## packaging of this file.  Please review the following information to
-## ensure the GNU Lesser General Public License version 2.1 requirements
-## will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-##
-## In addition, as a special exception, Digia gives you certain additional
-## rights.  These rights are described in the Digia Qt LGPL Exception
-## version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
-##
-#############################################################################
+############################################################################
+#
+# Copyright (C) 2016 The Qt Company Ltd.
+# Contact: https://www.qt.io/licensing/
+#
+# This file is part of Qt Creator.
+#
+# Commercial License Usage
+# Licensees holding valid commercial Qt licenses may use this file in
+# accordance with the commercial license agreement provided with the
+# Software or, alternatively, in accordance with the terms contained in
+# a written agreement between you and The Qt Company. For licensing terms
+# and conditions see https://www.qt.io/terms-conditions. For further
+# information use the contact form at https://www.qt.io/contact-us.
+#
+# GNU General Public License Usage
+# Alternatively, this file may be used under the terms of the GNU
+# General Public License version 3 as published by the Free Software
+# Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+# included in the packaging of this file. Please review the following
+# information to ensure the GNU General Public License requirements will
+# be met: https://www.gnu.org/licenses/gpl-3.0.html.
+#
+############################################################################
 
 import __builtin__
 import re
 
+def openQbsProject(projectPath):
+    cleanUpUserFiles(projectPath)
+    invokeMenuItem("File", "Open File or Project...")
+    selectFromFileDialog(projectPath)
+
 def openQmakeProject(projectPath, targets=Targets.desktopTargetClasses(), fromWelcome=False):
     cleanUpUserFiles(projectPath)
     if fromWelcome:
-        mouseClick(waitForObject(":OpenProject_QStyleItem"), 5, 5, 0, Qt.LeftButton)
-        if not platform.system() == "Darwin":
-            waitFor("waitForObject(':fileNameEdit_QLineEdit', 1000).focus == True", 3000)
+        welcomePage = ":Qt Creator.WelcomePage_QQuickWidget"
+        mouseClick(waitForObject("{clip='false' container='%s' enabled='true' text='Open Project' "
+                                 "type='Button' unnamed='1' visible='true'}" % welcomePage),
+                   5, 5, 0, Qt.LeftButton)
     else:
         invokeMenuItem("File", "Open File or Project...")
     selectFromFileDialog(projectPath)
@@ -54,40 +56,26 @@ def openQmakeProject(projectPath, targets=Targets.desktopTargetClasses(), fromWe
     return checkedTargets
 
 def openCmakeProject(projectPath, buildDir):
+    def additionalFunction():
+        pChooser = waitForObject("{leftWidget={text='Default' type='QCheckBox' unnamed='1' "
+                                 "visible='1'} type='Utils::PathChooser' unnamed='1' visible='1'}")
+        lineEdit = getChildByClass(pChooser, "Utils::FancyLineEdit")
+        replaceEditorContent(lineEdit, buildDir)
+        # disable all build configurations except "Default"
+        configs = ['Debug', 'Release', 'Release with Debug Information', 'Minimum Size Release']
+        for checkbox in configs:
+            ensureChecked(waitForObject("{text='%s' type='QCheckBox' unnamed='1' visible='1' "
+                                        "window=':Qt Creator_Core::Internal::MainWindow'}"
+                                        % checkbox), False)
+        ensureChecked(waitForObject("{text='Default' type='QCheckBox' unnamed='1' visible='1' "
+                      "window=':Qt Creator_Core::Internal::MainWindow'}"), True)
+
     invokeMenuItem("File", "Open File or Project...")
     selectFromFileDialog(projectPath)
-    replaceEditorContent("{type='Utils::BaseValidatingLineEdit' unnamed='1' visible='1'"
-                         "window=':CMake Wizard_CMakeProjectManager::Internal::CMakeOpenProjectWizard'}", buildDir)
-    clickButton(waitForObject(":CMake Wizard.Next_QPushButton"))
-    return __handleCmakeWizardPage__()
-
-def __handleCmakeWizardPage__():
-    generatorCombo = waitForObject(":Generator:_QComboBox")
-    mkspec = __getMkspecFromQmake__("qmake")
-    test.log("Using mkspec '%s'" % mkspec)
-
-    generatorText = "Unix Generator (Desktop 474 GCC)"
-    if "win32-" in mkspec:
-        generatorName = {"win32-g++" : "MinGW Generator (Desktop 474 GCC)",
-                         "win32-msvc2010" : "NMake Generator (Desktop 480 MSVC2010)"}
-        if mkspec in generatorName:
-            generatorText = generatorName[mkspec]
-    index = generatorCombo.findText(generatorText)
-    if index == -1:
-        test.warning("No matching CMake generator for mkspec '%s' found." % mkspec)
-    else:
-        generatorCombo.setCurrentIndex(index)
-
-    clickButton(waitForObject(":CMake Wizard.Run CMake_QPushButton"))
-    try:
-        clickButton(waitForObject(":CMake Wizard.Finish_QPushButton", 60000))
-    except LookupError:
-        cmakeOutput = waitForObject("{type='QPlainTextEdit' unnamed='1' visible='1' "
-                                    "window=':CMake Wizard_CMakeProjectManager::Internal::CMakeOpenProjectWizard'}")
-        test.warning("Error while executing cmake - see details for cmake output.",
-                     str(cmakeOutput.plainText))
-        clickButton(waitForObject(":CMake Wizard.Cancel_QPushButton"))
-        return False
+    __chooseTargets__(0) # uncheck all
+    __chooseTargets__(Targets.DESKTOP_480_DEFAULT, additionalFunc=additionalFunction)
+    clickButton(waitForObject("{text='Configure Project' type='QPushButton' unnamed='1' visible='1'"
+                              "window=':Qt Creator_Core::Internal::MainWindow'}"))
     return True
 
 # this function returns a list of available targets - this is not 100% error proof
@@ -96,10 +84,13 @@ def __handleCmakeWizardPage__():
 # this list can be used in __chooseTargets__()
 def __createProjectOrFileSelectType__(category, template, fromWelcome = False, isProject=True):
     if fromWelcome:
-        mouseClick(waitForObject(":CreateProject_QStyleItem"), 5, 5, 0, Qt.LeftButton)
+        welcomePage = ":Qt Creator.WelcomePage_QQuickWidget"
+        mouseClick(waitForObject("{clip='false' container='%s' enabled='true' text='New Project' "
+                                 "type='Button' unnamed='1' visible='true'}" % welcomePage),
+                   5, 5, 0, Qt.LeftButton)
     else:
         invokeMenuItem("File", "New File or Project...")
-    categoriesView = waitForObject("{type='QTreeView' name='templateCategoryView'}")
+    categoriesView = waitForObject(":New.templateCategoryView_QTreeView")
     if isProject:
         clickItem(categoriesView, "Projects." + category, 5, 5, 0, Qt.LeftButton)
     else:
@@ -110,11 +101,12 @@ def __createProjectOrFileSelectType__(category, template, fromWelcome = False, i
     clickButton(waitForObject("{text='Choose...' type='QPushButton' unnamed='1' visible='1'}"))
     return __getSupportedPlatforms__(str(text), template)[0]
 
-def __createProjectSetNameAndPath__(path, projectName = None, checks = True):
-    directoryEdit = waitForObject("{type='Utils::BaseValidatingLineEdit' unnamed='1' visible='1'}")
+def __createProjectSetNameAndPath__(path, projectName = None, checks = True, libType = None):
+    directoryEdit = waitForObject("{type='Utils::FancyLineEdit' unnamed='1' visible='1' "
+                                  "toolTip?='Full path: *'}")
     replaceEditorContent(directoryEdit, path)
     projectNameEdit = waitForObject("{name='nameLineEdit' visible='1' "
-                                    "type='Utils::ProjectNameValidatingLineEdit'}")
+                                    "type='Utils::FancyLineEdit'}")
     if projectName == None:
         projectName = projectNameEdit.text
     else:
@@ -125,30 +117,23 @@ def __createProjectSetNameAndPath__(path, projectName = None, checks = True):
         test.verify(labelCheck, "Project name and base directory without warning or error")
     # make sure this is not set as default location
     ensureChecked("{type='QCheckBox' name='projectsDirectoryCheckBox' visible='1'}", False)
+    if libType != None:
+        selectFromCombo(waitForObject("{leftWidget={text='Type' type='QLabel' unnamed='1' "
+                                      "visible='1'} type='QComboBox' unnamed='1' visible='1'}"),
+                        LibType.getStringForLib(libType))
     clickButton(waitForObject(":Next_QPushButton"))
     return str(projectName)
 
-def __createProjectHandleQtQuickSelection__(qtQuickVersion, withControls):
-    comboBox = waitForObject("{type='QComboBox' unnamed='1' visible='1' "
-                             "leftWidget={text='Qt Quick component set:' type='QLabel' unnamed='1' "
-                             "visible='1'}}")
-    if qtQuickVersion == 1:
-        selectFromCombo(comboBox, "Qt Quick 1.1")
-        if withControls:
-            test.warning("Controls are not available for Quick 1.")
-    elif qtQuickVersion == 2:
-        if withControls:
-            selectFromCombo(comboBox, "Qt Quick Controls 1.0")
-        else:
-            selectFromCombo(comboBox, "Qt Quick 2.0")
-    else:
-        test.fatal("Got unknown Qt Quick version: %s - trying to continue." % str(qtQuickVersion))
-    label = waitForObject("{type='QLabel' unnamed='1' visible='1' text?='Creates a *' }")
-    requires = re.match(".*Requires Qt (\d\.\d).*", str(label.text))
-    if requires:
-        requires = requires.group(1)
+def __createProjectHandleQtQuickSelection__(minimumQtVersion):
+    comboBox = waitForObject("{leftWidget=':Minimal required Qt version:_QLabel' name='QtVersion' "
+                             "type='Utils::TextFieldComboBox' visible='1'}")
+    try:
+        selectFromCombo(comboBox, "Qt %s" % minimumQtVersion)
+    except:
+        t,v = sys.exc_info()[:2]
+        test.fatal("Exception while trying to select Qt version", "%s (%s)" % (str(t), str(v)))
     clickButton(waitForObject(":Next_QPushButton"))
-    return requires
+    return minimumQtVersion
 
 # Selects the Qt versions for a project
 # param checks turns tests in the function on if set to True
@@ -156,15 +141,22 @@ def __createProjectHandleQtQuickSelection__(qtQuickVersion, withControls):
 def __selectQtVersionDesktop__(checks, available=None):
     checkedTargets = __chooseTargets__(Targets.desktopTargetClasses(), available)
     if checks:
-        cbObject = ("{type='QCheckBox' text='%s' unnamed='1' visible='1' "
-                    "container={type='Utils::DetailsWidget' visible='1' unnamed='1'}}")
-        verifyChecked(cbObject % "Debug")
-        verifyChecked(cbObject % "Release")
+        for target in checkedTargets:
+            detailsWidget = waitForObject("{type='Utils::DetailsWidget' unnamed='1' visible='1' "
+                                          "summaryText='%s'}" % Targets.getStringForTarget(target))
+            detailsButton = getChildByClass(detailsWidget, "Utils::DetailsButton")
+            if test.verify(detailsButton != None, "Verifying if 'Details' button could be found"):
+                clickButton(detailsButton)
+                cbObject = ("{type='QCheckBox' text='%s' unnamed='1' visible='1' "
+                            "container=%s}")
+                verifyChecked(cbObject % ("Debug", objectMap.realName(detailsWidget)))
+                verifyChecked(cbObject % ("Release", objectMap.realName(detailsWidget)))
+                clickButton(detailsButton)
     clickButton(waitForObject(":Next_QPushButton"))
     return checkedTargets
 
-def __createProjectHandleLastPage__(expectedFiles = None, addToVersionControl = "<None>", addToProject = None):
-    if expectedFiles != None:
+def __createProjectHandleLastPage__(expectedFiles=[], addToVersionControl="<None>", addToProject=None):
+    if len(expectedFiles):
         summary = waitForObject("{name='filesLabel' text?='<qt>Files to be added in<pre>*</pre>' "
                                 "type='QLabel' visible='1'}").text
         verifyItemOrder(expectedFiles, summary)
@@ -209,7 +201,7 @@ def __modifyAvailableTargets__(available, requiredQt, asStrings=False):
 # param checks turns tests in the function on if set to True
 def createProject_Qt_GUI(path, projectName, checks = True, addToVersionControl = "<None>"):
     template = "Qt Widgets Application"
-    available = __createProjectOrFileSelectType__("  Applications", template)
+    available = __createProjectOrFileSelectType__("  Application", template)
     __createProjectSetNameAndPath__(path, projectName, checks)
     checkedTargets = __selectQtVersionDesktop__(checks, available)
 
@@ -230,7 +222,7 @@ def createProject_Qt_GUI(path, projectName, checks = True, addToVersionControl =
 
     clickButton(waitForObject(":Next_QPushButton"))
 
-    expectedFiles = None
+    expectedFiles = []
     if checks:
         if platform.system() in ('Windows', 'Microsoft'):
             path = os.path.abspath(path)
@@ -240,7 +232,8 @@ def createProject_Qt_GUI(path, projectName, checks = True, addToVersionControl =
     __createProjectHandleLastPage__(expectedFiles, addToVersionControl)
 
     progressBarWait(20000)
-    __verifyFileCreation__(path, expectedFiles)
+    if checks:
+        __verifyFileCreation__(path, expectedFiles)
     return checkedTargets
 
 # Creates a Qt Console project
@@ -248,11 +241,11 @@ def createProject_Qt_GUI(path, projectName, checks = True, addToVersionControl =
 # param projectName is the name for the new project
 # param checks turns tests in the function on if set to True
 def createProject_Qt_Console(path, projectName, checks = True):
-    available = __createProjectOrFileSelectType__("  Applications", "Qt Console Application")
+    available = __createProjectOrFileSelectType__("  Application", "Qt Console Application")
     __createProjectSetNameAndPath__(path, projectName, checks)
     checkedTargets = __selectQtVersionDesktop__(checks, available)
 
-    expectedFiles = None
+    expectedFiles = []
     if checks:
         if platform.system() in ('Windows', 'Microsoft'):
             path = os.path.abspath(path)
@@ -264,35 +257,49 @@ def createProject_Qt_Console(path, projectName, checks = True):
     __createProjectHandleLastPage__(expectedFiles)
 
     progressBarWait(10000)
-    __verifyFileCreation__(path, expectedFiles)
+    if checks:
+        __verifyFileCreation__(path, expectedFiles)
     return checkedTargets
 
 def createNewQtQuickApplication(workingDir, projectName = None,
-                                targets=Targets.desktopTargetClasses(), qtQuickVersion=1,
-                                fromWelcome=False, withControls=False):
-    available = __createProjectOrFileSelectType__("  Applications", "Qt Quick Application", fromWelcome)
+                                targets=Targets.desktopTargetClasses(), minimumQtVersion="5.3",
+                                withControls = False, fromWelcome=False):
+    if withControls:
+        template = "Qt Quick Controls Application"
+    else:
+        template = "Qt Quick Application"
+    available = __createProjectOrFileSelectType__("  Application", template, fromWelcome)
     projectName = __createProjectSetNameAndPath__(workingDir, projectName)
-    requiredQt = __createProjectHandleQtQuickSelection__(qtQuickVersion, withControls)
+    requiredQt = __createProjectHandleQtQuickSelection__(minimumQtVersion)
     __modifyAvailableTargets__(available, requiredQt)
     checkedTargets = __chooseTargets__(targets, available)
     snooze(1)
-    clickButton(waitForObject(":Next_QPushButton"))
-    __createProjectHandleLastPage__()
+    if len(checkedTargets):
+        clickButton(waitForObject(":Next_QPushButton"))
+        __createProjectHandleLastPage__()
+        progressBarWait(10000)
+    else:
+        clickButton(waitForObject("{type='QPushButton' text='Cancel' visible='1'}"))
 
-    progressBarWait(10000)
     return checkedTargets, projectName
 
-def createNewQtQuickUI(workingDir, qtQuickVersion=1, withControls=False):
-    __createProjectOrFileSelectType__("  Applications", "Qt Quick UI")
+def createNewQtQuickUI(workingDir, qtVersion = "5.3", withControls = False):
+    if withControls:
+        template = 'Qt Quick Controls UI'
+    else:
+        template = 'Qt Quick UI'
+    __createProjectOrFileSelectType__("  Other Project", template)
     if workingDir == None:
         workingDir = tempDir()
     projectName = __createProjectSetNameAndPath__(workingDir)
-    __createProjectHandleQtQuickSelection__(qtQuickVersion, withControls)
+    __createProjectHandleQtQuickSelection__(qtVersion)
     __createProjectHandleLastPage__()
+    progressBarWait(10000)
+
     return projectName
 
 def createNewQmlExtension(workingDir, targets=Targets.DESKTOP_474_GCC, qtQuickVersion=1):
-    available = __createProjectOrFileSelectType__("  Libraries", "Qt Quick %d Extension Plugin"
+    available = __createProjectOrFileSelectType__("  Library", "Qt Quick %d Extension Plugin"
                                                   % qtQuickVersion)
     if workingDir == None:
         workingDir = tempDir()
@@ -311,7 +318,7 @@ def createNewQmlExtension(workingDir, targets=Targets.DESKTOP_474_GCC, qtQuickVe
     return checkedTargets
 
 def createEmptyQtProject(workingDir=None, projectName=None, targets=Targets.desktopTargetClasses()):
-    __createProjectOrFileSelectType__("  Other Project", "Empty Qt Project")
+    __createProjectOrFileSelectType__("  Other Project", "Empty qmake Project")
     if workingDir == None:
         workingDir = tempDir()
     projectName = __createProjectSetNameAndPath__(workingDir, projectName)
@@ -322,52 +329,99 @@ def createEmptyQtProject(workingDir=None, projectName=None, targets=Targets.desk
     return projectName, checkedTargets
 
 def createNewNonQtProject(workingDir=None, projectName=None, target=Targets.DESKTOP_474_GCC,
-                          plainC=False, cmake=False):
+                          plainC=False, cmake=False, qbs=False):
     if plainC:
-        template = "Plain C Project"
+        template = "Plain C Application"
     else:
-        template = "Plain C++ Project"
-    if cmake:
-        template += " (CMake Build)"
+        template = "Plain C++ Application"
+
     available = __createProjectOrFileSelectType__("  Non-Qt Project", template)
     if workingDir == None:
         workingDir = tempDir()
     projectName = __createProjectSetNameAndPath__(workingDir, projectName)
-    if cmake:
-        __createProjectHandleLastPage__()
-        clickButton(waitForObject(":Next_QPushButton"))
-        if not __handleCmakeWizardPage__():
-            return None
-    else:
-        __chooseTargets__(target, availableTargets=available)
-        clickButton(waitForObject(":Next_QPushButton"))
-        __createProjectHandleLastPage__()
+
+    buildSystem = "qmake"
+    if qbs:
+        buildSystem = "Qbs"
+        if cmake:
+            test.warning("Unsupported combination, at least one of parameters cmake and qbs must "
+                         "be False, ignoring the value of cmake")
+    elif cmake:
+        buildSystem = "CMake"
+    selectFromCombo("{name='BuildSystem' type='Utils::TextFieldComboBox' visible='1'}", buildSystem)
+    clickButton(waitForObject(":Next_QPushButton"))
+
+    __chooseTargets__(target, availableTargets=available)
+    clickButton(waitForObject(":Next_QPushButton"))
+    __createProjectHandleLastPage__()
     return projectName
 
+def createNewCPPLib(projectDir = None, projectName = None, className = None, fromWelcome = False,
+                    target = Targets.DESKTOP_474_GCC, isStatic = False, modules = ["QtCore"]):
+    available = __createProjectOrFileSelectType__("  Library", "C++ Library", fromWelcome, True)
+    if isStatic:
+        libType = LibType.STATIC
+    else:
+        libType = LibType.SHARED
+    if projectDir == None:
+        projectDir = tempDir()
+    projectName = __createProjectSetNameAndPath__(projectDir, projectName, False, libType)
+    checkedTargets = __chooseTargets__(target, available)
+    snooze(1)
+    clickButton(waitForObject(":Next_QPushButton"))
+    __createProjectHandleModuleSelection__(modules)
+    className = __createProjectHandleClassInformation__(className)
+    __createProjectHandleLastPage__()
+    return checkedTargets, projectName, className
+
+def createNewQtPlugin(projectDir=None, projectName=None, className=None, fromWelcome=False,
+                      target=Targets.DESKTOP_474_GCC, baseClass="QGenericPlugin"):
+    available = __createProjectOrFileSelectType__("  Library", "C++ Library", fromWelcome, True)
+    if projectDir == None:
+        projectDir = tempDir()
+    projectName = __createProjectSetNameAndPath__(projectDir, projectName, False, LibType.QT_PLUGIN)
+    checkedTargets = __chooseTargets__(target, available)
+    snooze(1)
+    clickButton(waitForObject(":Next_QPushButton"))
+    className = __createProjectHandleClassInformation__(className, baseClass)
+    __createProjectHandleLastPage__()
+    return checkedTargets, projectName, className
+
 # parameter target can be an OR'd value of Targets
 # parameter availableTargets should be the result of __createProjectOrFileSelectType__()
 #           or use None as a fallback
-def __chooseTargets__(targets=Targets.DESKTOP_474_GCC, availableTargets=None):
+# parameter additionalFunc function to be executed inside the detailed view of each chosen kit
+#           if present, 'Details' button will be clicked, function will be executed,
+#           'Details' button will be clicked again
+def __chooseTargets__(targets=Targets.DESKTOP_474_GCC, availableTargets=None, additionalFunc=None):
     if availableTargets != None:
         available = availableTargets
     else:
         # following targets depend on the build environment - added for further/later tests
-        available = [Targets.DESKTOP_474_GCC, Targets.DESKTOP_480_GCC, Targets.DESKTOP_501_DEFAULT,
-                     Targets.MAEMO5, Targets.EMBEDDED_LINUX, Targets.SIMULATOR, Targets.HARMATTAN]
+        available = list(Targets.ALL_TARGETS)
         if platform.system() in ('Windows', 'Microsoft'):
             available.remove(Targets.EMBEDDED_LINUX)
-            available.append(Targets.DESKTOP_480_MSVC2010)
-    for target in filter(lambda x: x in available,
-                         (Targets.MAEMO5, Targets.HARMATTAN)):
-        available.remove(target)
+        elif platform.system() == 'Darwin':
+            available.remove(Targets.DESKTOP_541_GCC)
     checkedTargets = []
     for current in available:
         mustCheck = targets & current == current
         try:
             ensureChecked("{type='QCheckBox' text='%s' visible='1'}" % Targets.getStringForTarget(current),
                           mustCheck, 3000)
-            if (mustCheck):
+            if mustCheck:
                 checkedTargets.append(current)
+
+                # perform additional function on detailed kits view
+                if additionalFunc:
+                    detailsWidget = waitForObject("{type='Utils::DetailsWidget' unnamed='1' "
+                                                  "window=':Qt Creator_Core::Internal::MainWindow' "
+                                                  "toolTip?='<html><body><h3>%s</h3>*' visible='1'}"
+                                                  % Targets.getStringForTarget(current))
+                    detailsButton = getChildByClass(detailsWidget, "Utils::DetailsButton")
+                    clickButton(detailsButton)
+                    additionalFunc()
+                    clickButton(detailsButton)
         except LookupError:
             if mustCheck:
                 test.fail("Failed to check target '%s'." % Targets.getStringForTarget(current))
@@ -377,6 +431,37 @@ def __chooseTargets__(targets=Targets.DESKTOP_474_GCC, availableTargets=None):
                     test.warning("Target '%s' is not set up correctly." % Targets.getStringForTarget(current))
     return checkedTargets
 
+def __createProjectHandleModuleSelection__(modules):
+    modulesPage = waitForObject("{type='QmakeProjectManager::Internal::ModulesPage' unnamed='1' "
+                                "visible='1'}")
+    chckBoxes = filter(lambda x: className(x) == 'QCheckBox', object.children(modulesPage))
+    chckBoxLabels = set([str(cb.text) for cb in chckBoxes])
+    if not set(modules).issubset(chckBoxLabels):
+        test.fatal("You want to check module(s) not available at 'Module Selection' page.",
+                   "Not available: %s" % str(set(modules).difference(chckBoxLabels)))
+    for checkBox in chckBoxes:
+        test.log("(Un)Checking module checkbox '%s'" % str(checkBox.text))
+        ensureChecked(checkBox, str(checkBox.text) in modules, 3000)
+    clickButton(waitForObject(":Next_QPushButton"))
+
+def __createProjectHandleClassInformation__(className, baseClass=None):
+    if baseClass:
+        selectFromCombo("{name='baseClassComboBox' type='QComboBox' visible='1'}", baseClass)
+    classLineEd = waitForObject("{name='classLineEdit' type='Utils::ClassNameValidatingLineEdit' "
+                                "visible='1'}")
+    result = str(classLineEd.text)
+    if className:
+        replaceEditorContent(classLineEd, className)
+    try:
+        waitForObject("{text='The class name contains invalid characters.' type='QLabel' "
+                     "unnamed='1' visible='1'}", 1000)
+        test.fatal("Class name contains invalid characters - using default.")
+        replaceEditorContent(classLineEd, result)
+    except:
+        result = className
+    clickButton(waitForObject(":Next_QPushButton"))
+    return result
+
 def waitForProcessRunning(running=True):
     outputButton = waitForObject(":Qt Creator_AppOutput_Core::Internal::OutputPaneToggleButton")
     if not waitFor("outputButton.checked", 10000):
@@ -398,7 +483,7 @@ def waitForProcessRunning(running=True):
 # by yourself (or use the function parameter)
 # ATTENTION! Make sure this function won't fail and the sub-process will end when the function returns
 # returns None if the build failed, False if the subprocess did not start, and True otherwise
-def runAndCloseApp(withHookInto=False, executable=None, port=None, function=None, sType=None, userDefinedType=None, quickVersion=1):
+def runAndCloseApp(withHookInto=False, executable=None, port=None, function=None, sType=None, userDefinedType=None, quickVersion="1.1"):
     runButton = waitForObject(":*Qt Creator.Run_Core::Internal::FancyToolButton")
     clickButton(runButton)
     if sType != SubprocessType.QT_QUICK_UI:
@@ -465,7 +550,8 @@ def __closeSubprocessByHookingInto__(executable, port, function, sType, userDefT
         attachToApplication(executable)
     except:
         resetApplicationContextToCreator()
-        if "Loading Qt Wrapper failed" in str(output.plainText):
+        if ("Loading Qt Wrapper failed" in str(output.plainText)
+            or "Failed to assign process to job object" in str(output.plainText)):
             test.warning("Loading of Qt Wrapper failed - probably different Qt versions.",
                          "Resetting hook-into settings to continue.")
             # assuming we're still on the build settings of the current project (TODO)
@@ -543,50 +629,69 @@ def __getSupportedPlatforms__(text, templateName, getAsStrings=False):
         version = res.group("version")
     else:
         version = None
-    if 'Supported Platforms' in text:
+    if 'only available with Qt 5.6' in text:
+        result = [Targets.DESKTOP_561_DEFAULT]
+    elif 'Supported Platforms' in text:
         supports = text[text.find('Supported Platforms'):].split(":")[1].strip().split(" ")
         result = []
         if 'Desktop' in supports:
             if version == None or version < "5.0":
                 result.append(Targets.DESKTOP_474_GCC)
-                result.append(Targets.DESKTOP_480_GCC)
+                result.append(Targets.DESKTOP_480_DEFAULT)
                 if platform.system() in ("Linux", "Darwin"):
                     result.append(Targets.EMBEDDED_LINUX)
-                elif platform.system() in ('Windows', 'Microsoft'):
-                    result.append(Targets.DESKTOP_480_MSVC2010)
-            result.append(Targets.DESKTOP_501_DEFAULT)
-        if 'MeeGo/Harmattan' in supports:
-            result.append(Targets.HARMATTAN)
-        if 'Maemo/Fremantle' in supports:
-            result.append(Targets.MAEMO5)
-        if not ("BlackBerry" in templateName or re.search("custom Qt Creator plugin", text)) and (version == None or version < "5.0"):
+            result.extend([Targets.DESKTOP_531_DEFAULT, Targets.DESKTOP_561_DEFAULT])
+            if platform.system() != 'Darwin':
+                result.append(Targets.DESKTOP_541_GCC)
+        if not templateName == "Qt Creator Plugin" and (version == None or version < "5.0"):
             result.append(Targets.SIMULATOR)
     elif 'Platform independent' in text:
-        # MAEMO5 and HARMATTAN could be wrong here - depends on having Madde plugin enabled or not
-        result = [Targets.DESKTOP_474_GCC, Targets.DESKTOP_480_GCC, Targets.DESKTOP_501_DEFAULT,
-                  Targets.MAEMO5, Targets.SIMULATOR, Targets.HARMATTAN]
-        if platform.system() in ('Windows', 'Microsoft'):
-            result.append(Targets.DESKTOP_480_MSVC2010)
+        result = list(Targets.ALL_TARGETS)
+        result.remove(Targets.EMBEDDED_LINUX)
+        if platform.system() == 'Darwin':
+            result.remove(Targets.DESKTOP_541_GCC)
     else:
         test.warning("Returning None (__getSupportedPlatforms__())",
                      "Parsed text: '%s'" % text)
-        return None, None
+        return [], None
     if getAsStrings:
         result = Targets.getTargetsAsStrings(result)
     return result, version
 
 # copy example project (sourceExample is path to project) to temporary directory inside repository
-def prepareTemplate(sourceExample):
+def prepareTemplate(sourceExample, deploymentDir=None):
     templateDir = os.path.abspath(tempDir() + "/template")
     try:
         shutil.copytree(sourceExample, templateDir)
     except:
         test.fatal("Error while copying '%s' to '%s'" % (sourceExample, templateDir))
         return None
+    if deploymentDir:
+        shutil.copytree(os.path.abspath(sourceExample + deploymentDir),
+                        os.path.abspath(templateDir + deploymentDir))
     return templateDir
 
+# check and copy files of given dataset to an existing templateDir
+def checkAndCopyFiles(dataSet, fieldName, templateDir):
+    files = map(lambda record:
+                os.path.normpath(os.path.join(srcPath, testData.field(record, fieldName))),
+                dataSet)
+    for currentFile in files:
+        if not neededFilePresent(currentFile):
+            return []
+    return copyFilesToDir(files, templateDir)
+
+# copy a list of files to an existing targetDir
+def copyFilesToDir(files, targetDir):
+    result = []
+    for filepath in files:
+        dst = os.path.join(targetDir, os.path.basename(filepath))
+        shutil.copyfile(filepath, dst)
+        result.append(dst)
+    return result
+
 def __sortFilenamesOSDependent__(filenames):
-    if platform.system() in ('Windows', 'Microsoft'):
+    if platform.system() in ('Windows', 'Microsoft', 'Darwin'):
         filenames.sort(key=str.lower)
     else:
         filenames.sort()
@@ -629,22 +734,54 @@ def compareProjectTree(rootObject, dataset):
             return
     test.passes("No errors found in project tree")
 
-def addCPlusPlusFileToCurrentProject(name, template, forceOverwrite=False, addToVCS = "<None>"):
+# creates C++ file(s) and adds them to the current project if one is open
+# name                  name of the created object: filename for files, classname for classes
+# template              "C++ Class", "C++ Header File" or "C++ Source File"
+# forceOverwrite        bool: force overwriting existing files?
+# addToVCS              name of VCS to add the file(s) to
+# newBasePath           path to create the file(s) at
+# expectedSourceName    expected name of created source file
+# expectedHeaderName    expected name of created header file
+def addCPlusPlusFileToCurrentProject(name, template, forceOverwrite=False, addToVCS="<None>",
+                                     newBasePath=None, expectedSourceName=None, expectedHeaderName=None):
     if name == None:
         test.fatal("File must have a name - got None.")
         return
     __createProjectOrFileSelectType__("  C++", template, isProject=False)
-    window = "{type='Utils::FileWizardDialog' unnamed='1' visible='1'}"
-    basePath = str(waitForObject("{type='Utils::BaseValidatingLineEdit' unnamed='1' visible='1' "
-                                 "window=%s}" % window).text)
-    lineEdit = waitForObject("{name='nameLineEdit' type='Utils::FileNameValidatingLineEdit' "
-                             "visible='1' window=%s}" % window)
+    window = "{type='ProjectExplorer::JsonWizard' unnamed='1' visible='1'}"
+    basePathEdit = waitForObject("{type='Utils::FancyLineEdit' unnamed='1' visible='1' "
+                                 "window=%s}" % window)
+    if newBasePath:
+        replaceEditorContent(basePathEdit, newBasePath)
+    basePath = str(basePathEdit.text)
+    lineEdit = None
+    if template == "C++ Class":
+        lineEdit = waitForObject("{name='Class' type='QLineEdit' visible='1'}")
+    else:
+        lineEdit = waitForObject("{name='nameLineEdit' type='Utils::FileNameValidatingLineEdit' "
+                                 "visible='1' window=%s}" % window)
     replaceEditorContent(lineEdit, name)
+    expectedFiles = []
+    if expectedSourceName:
+        expectedFiles += [expectedSourceName]
+        if template == "C++ Class":
+            test.compare(str(waitForObject("{name='SrcFileName' type='QLineEdit' visible='1'}").text),
+                         expectedSourceName)
+    if expectedHeaderName:
+        expectedFiles += [expectedHeaderName]
+        if template == "C++ Class":
+            test.compare(str(waitForObject("{name='HdrFileName' type='QLineEdit' visible='1'}").text),
+                         expectedHeaderName)
     clickButton(waitForObject(":Next_QPushButton"))
-    fileExistedBefore = os.path.exists(os.path.join(basePath, name))
-    __createProjectHandleLastPage__(addToVersionControl = addToVCS)
+    fileExistedBefore = False
+    if template == "C++ Class":
+        fileExistedBefore = (os.path.exists(os.path.join(basePath, name.lower() + ".cpp"))
+                             or os.path.exists(os.path.join(basePath, name.lower() + ".h")))
+    else:
+        fileExistedBefore = os.path.exists(os.path.join(basePath, name))
+    __createProjectHandleLastPage__(expectedFiles, addToVersionControl=addToVCS)
     if (fileExistedBefore):
-        overwriteDialog = "{type='Core::Internal::PromptOverwriteDialog' unnamed='1' visible='1'}"
+        overwriteDialog = "{type='Core::PromptOverwriteDialog' unnamed='1' visible='1'}"
         waitForObject(overwriteDialog)
         if forceOverwrite:
             buttonToClick = 'OK'
@@ -652,13 +789,3 @@ def addCPlusPlusFileToCurrentProject(name, template, forceOverwrite=False, addTo
             buttonToClick = 'Cancel'
         clickButton("{text='%s' type='QPushButton' unnamed='1' visible='1' window=%s}"
                     % (buttonToClick, overwriteDialog))
-
-def qt5SDKPath():
-    if platform.system() in ('Microsoft', 'Windows'):
-        return os.path.abspath("C:/Qt/Qt5.0.1/5.0.1/msvc2010")
-    elif platform.system() == 'Linux':
-        if __is64BitOS__():
-            return os.path.expanduser("~/Qt5.0.1/5.0.1/gcc_64")
-        return os.path.expanduser("~/Qt5.0.1/5.0.1/gcc")
-    else:
-        return os.path.expanduser("~/Qt5.0.1/5.0.1/clang_64")
